@@ -129,11 +129,27 @@ cap_init(const char *device, const char *filter)
    if (ioctl(pcap_fd, FIONBIO, &one) == -1)
       err(1, "ioctl(pcap_fd, FIONBIO)"); }
 
-{ struct timeval t = { 0, TIMEOUT * 1000 }; /* msec -> usec */
+{ struct timeval t = { 0, CAP_TIMEOUT * 1000 }; /* msec -> usec */
    if (ioctl(pcap_fd, BIOCSRTIMEOUT, &t) == -1)
       err(1, "ioctl(pcap_fd, BIOCSRTIMEOUT)"); }
 #endif
 
+#ifdef BIOCSETWF
+{
+   /* Deny all writes to the socket */
+   struct bpf_insn bpf_wfilter[] = { BPF_STMT(BPF_RET+BPF_K, 0) };
+   int wf_len = sizeof(bpf_wfilter) / sizeof(struct bpf_insn);
+   struct bpf_program pr;
+
+   pr.bf_len = wf_len;
+   pr.bf_insns = bpf_wfilter;
+
+   if (ioctl(pcap_fd, BIOCSETWF, &pr) == -1)
+      err(1, "ioctl(pcap_fd, BIOCSETFW)");
+   verbosef("filtered out BPF writes");
+}
+#endif
+
 #ifdef BIOCLOCK
    if (ioctl(pcap_fd, BIOCLOCK) == -1)
       err(1, "ioctl(pcap_fd, BIOCLOCK)");
