@@ -253,8 +253,10 @@ privdrop(const int want_chroot)
       err(1, "getpwnam(" PRIVDROP_USER ")");
    if (want_chroot) {
       tzset(); /* read /etc/localtime before we chroot */
+      if (chdir(CHROOT_DIR) == -1)
+         err(1, "chdir(" CHROOT_DIR ") failed");
       if (chroot(CHROOT_DIR) == -1)
-         err(1, "chroot(" CHROOT_DIR ")");
+         err(1, "chroot(" CHROOT_DIR ") failed");
    }
    if (setgid(pw->pw_gid) == -1)
       err(1, "setgid");
