@@ -34,7 +34,12 @@
 #  include <sys/ethernet.h>
 #  define ETHER_HDR_LEN 14
 # else
-#  include <net/ethernet.h>
+#  ifdef _AIX
+#   include <netinet/if_ether.h>
+#   define ETHER_HDR_LEN 14
+#  else
+#   include <net/ethernet.h>
+#  endif
 # endif
 #endif
 
@@ -51,6 +56,8 @@ static void decode_loop(u_char *, const struct pcap_pkthdr *,
    const u_char *);
 static void decode_pppoe(u_char *, const struct pcap_pkthdr *,
    const u_char *);
+static void decode_linux_sll(u_char *, const struct pcap_pkthdr *,
+   const u_char *);
 static void decode_ip(const u_char *pdata, const uint32_t len,
    pktsummary *sm);
 
@@ -63,6 +70,7 @@ static const linkhdr_t linkhdrs[] = {
    { DLT_PPP,       PPP_HDR_LEN,   NULL },
    { DLT_FDDI,      FDDI_HDR_LEN,  NULL },
    { DLT_PPP_ETHER, PPPOE_HDR_LEN, decode_pppoe },
+   { DLT_LINUX_SLL, SLL_HDR_LEN,   decode_linux_sll },
    { -1, -1, NULL }
 };
 
@@ -198,6 +206,44 @@ decode_pppoe(u_char *user _unused_,
       verbosef("non-session PPPoE packet; ignoring.");
 }
 
+/* very similar to decode_ether ... */
+static void
+decode_linux_sll(u_char *user _unused_,
+      const struct pcap_pkthdr *pheader,
+      const u_char *pdata)
+{
+   const struct sll_header {
+      uint16_t packet_type;
+      uint16_t device_type;
+      uint16_t addr_length;
+#define SLL_MAX_ADDRLEN 8
+      uint8_t addr[SLL_MAX_ADDRLEN];
+      uint16_t ether_type;
+   } *hdr = (const struct sll_header *)pdata;
+   u_short type;
+   pktsummary sm;
+   memset(&sm, 0, sizeof(sm));
+
+   if (pheader->caplen < SLL_HDR_LEN) {
+      verbosef("linux_sll: packet too short (%u bytes)", pheader->caplen);
+      return;
+   }
+
+   type = ntohs( hdr->ether_type );
+   switch (type) {
+   case ETHERTYPE_IP:
+      decode_ip(pdata + SLL_HDR_LEN, pheader->caplen - SLL_HDR_LEN, &sm);
+      sm.time = pheader->ts.tv_sec;
+      acct_for(&sm);
+      break;
+   case ETHERTYPE_ARP:
+      /* known protocol, don't complain about it. */
+      break;
+   default:
+      verbosef("linux_sll: unknown protocol (%04x)", type);
+   }
+}
+
 static void
 decode_ip(const u_char *pdata, const uint32_t len, pktsummary *sm)
 {
