@@ -16,6 +16,7 @@
 #include "queue.h"
 #include "tree.h"
 
+#include <sys/param.h>
 #include <sys/socket.h>
 #include <sys/wait.h>
 #include <assert.h>
@@ -33,14 +34,10 @@ static void dns_main(void); /* this is what the child process runs */
 static int sock[2];
 static pid_t pid = -1;
 
-#define DNS_REPLY_LEN 1024
-union dns_reply {
-   char buf[DNS_REPLY_LEN];
-   struct {
-      in_addr_t ip;
-      int error;              /* h_errno, or 0 if no error */
-      char name[];            /* the rest of the buffer is the name */
-   } hdr;
+struct dns_reply {
+   in_addr_t ip;
+   int error; /* h_errno, or 0 if no error */
+   char name[MAXHOSTNAMELEN];
 };
 
 void
@@ -112,11 +109,12 @@ dns_queue(const in_addr_t ip)
    rec = xmalloc(sizeof(*rec));
    rec->ip = ip;
    if (RB_INSERT(tree_t, &ip_tree, rec) != NULL) {
-      /* already queued */
+      /* Already queued - this happens seldom enough that we don't care about
+       * the performance hit of needlessly malloc()ing. */
+      verbosef("already queued %s", ip_to_str(ip));
       free(rec);
       return;
    }
-   /* FIXME: else: we never free this from ip_tree */
 
    num_w = write(sock[PARENT], &ip, sizeof(ip)); /* won't block */
    if (num_w == 0)
@@ -127,6 +125,20 @@ dns_queue(const in_addr_t ip)
       err(1, "dns_queue: wrote %d instead of %d", num_w, sizeof(ip));
 }
 
+static void
+dns_unqueue(const in_addr_t ip)
+{
+   struct tree_rec tmp, *rec;
+
+   tmp.ip = ip;
+   if ((rec = RB_FIND(tree_t, &ip_tree, &tmp)) != NULL) {
+      RB_REMOVE(tree_t, &ip_tree, rec);
+      free(rec);
+   }
+   else
+      verbosef("couldn't unqueue %s - not in queue!", ip_to_str(ip));
+}
+
 /*
  * Returns non-zero if result waiting, stores IP and name into given pointers
  * (name buffer is allocated by dns_poll)
@@ -134,7 +146,7 @@ dns_queue(const in_addr_t ip)
 static int
 dns_get_result(in_addr_t *ip, char **name)
 {
-   union dns_reply reply;
+   struct dns_reply reply;
    ssize_t numread;
 
    numread = read(sock[PARENT], &reply, sizeof(reply));
@@ -151,11 +163,12 @@ dns_get_result(in_addr_t *ip, char **name)
          sizeof(reply), numread);
 
    /* Return successful reply. */
-   *ip = reply.hdr.ip;
-   if (reply.hdr.error != 0)
-      xasprintf(name, "(%s)", hstrerror(reply.hdr.error));
+   *ip = reply.ip;
+   if (reply.error != 0)
+      xasprintf(name, "(%s)", hstrerror(reply.error));
    else
-      *name = xstrdup(reply.hdr.name);
+      *name = xstrdup(reply.name);
+   dns_unqueue(reply.ip);
    return (1);
 
 error:
@@ -280,30 +293,27 @@ dns_main(void)
 
       /* Process queue. */
       if (dequeue(&ip)) {
-         union dns_reply reply;
+         struct dns_reply reply;
          struct hostent *he;
 
-         reply.hdr.ip = ip;
+         reply.ip = ip;
          he = gethostbyaddr((char *)&ip, sizeof(ip), AF_INET);
 
          /* On some platforms (for example Linux with GLIBC 2.3.3), h_errno
           * will be non-zero here even though the lookup succeeded.
           */
          if (he == NULL) {
-            reply.hdr.name[0] = '\0';
-            reply.hdr.error = h_errno;
+            reply.name[0] = '\0';
+            reply.error = h_errno;
          } else {
-            char *start = reply.hdr.name;
-            char *end = reply.buf + sizeof(reply);
-            size_t maxlen = end - start;
-
-            strlcpy(reply.hdr.name, he->h_name, maxlen);
-            reply.hdr.error = 0;
+            assert(sizeof(reply.name) > sizeof(char *)); /* not just a ptr */
+            strlcpy(reply.name, he->h_name, sizeof(reply.name));
+            reply.error = 0;
          }
          fd_set_block(sock[CHILD]);
          xwrite(sock[CHILD], &reply, sizeof(reply));
          verbosef("DNS: %s is %s", ip_to_str(ip),
-            (h_errno == 0)?reply.hdr.name:hstrerror(h_errno));
+            (h_errno == 0)?reply.name:hstrerror(h_errno));
       }
    }
 }
