@@ -221,8 +221,8 @@ html_front_page(void)
 
    str_appendf(buf,
       "<p>\n"
-       "<b>Total packets:</b> %'qu<br/>\n"
-       "<b>Total bytes:</b> %'qu<br/>\n"
+       "<b>Total packets:</b> <span id=\"tp\">%'qu</span><br/>\n"
+       "<b>Total bytes:</b> <span id=\"tb\">%'qu</span><br/>\n"
       "</p>\n",
       total_packets, total_bytes);
 
@@ -274,8 +274,10 @@ xml_graphs(void)
    unsigned int i, j;
    struct str *buf = str_make();
 
-   str_append(buf, "<graphs>\n");
-   for (i=0; i<(int)graph_db_size; i++) {
+   str_appendf(buf, "<graphs tp=\"%'qu\" tb=\"%'qu\">\n",
+      total_packets, total_bytes);
+
+   for (i=0; i<graph_db_size; i++) {
       const struct graph *g = graph_db[i];
 
       str_appendf(buf, "<%s>\n", g->unit);
