@@ -255,6 +255,10 @@ format_row_host(struct str *buf, const struct bucket *b)
       ip, ip, (b->u.host.dns == NULL)?"":b->u.host.dns,
       b->in, b->out, b->total
    );
+
+   /* Only resolve hosts "on demand" */
+   if (b->u.host.dns == NULL)
+      dns_queue(b->u.host.ip);
 }
 
 static void
@@ -533,15 +537,7 @@ host_free(struct host *h)
 struct bucket *
 host_get(const in_addr_t ip)
 {
-#if 0
    return (hashtable_find_or_insert(hosts_db, &ip));
-#else
-   /* FIXME: this may not be the best of ideas */
-   struct bucket *b = hashtable_find_or_insert(hosts_db, &ip);
-   if (b->u.host.dns == NULL)
-      dns_queue(b->u.host.ip);
-   return (b);
-#endif
 }
 
 /* ---------------------------------------------------------------------------
