@@ -19,12 +19,12 @@ static const char html_header_2[] =
 "<body>\n"
 "<h1 class=\"watermark\">darkstat3</h1>\n"
 "<div class=\"menu\">\n"
-"<ul class=\"menu\">\n"
-" <li><a href=\"http://purl.org/net/darkstat\">"
-      PACKAGE_STRING "</a></li>\n"
-" <li><a href=\"/\">graphs</a></li>\n"
-" <li><a href=\"/hosts/\">hosts</a></li>\n"
-"</ul>\n"
+ "<ul class=\"menu\">\n"
+  "<li class=\"label\">" PACKAGE_STRING "</li>"
+  "<li><a href=\"/\">graphs</a></li>"
+  "<li><a href=\"/hosts/\">hosts</a></li>"
+  "<li><a href=\"http://purl.org/net/darkstat\">homepage</a></li>"
+ "</ul>\n"
 "</div>\n"
 "<div class=\"content\">\n";
 
