@@ -118,8 +118,10 @@ static char *urldecode(const char *url)
         }
     }
     out[pos] = 0;
-
+#if 0
+    /* don't really need to realloc here - it's probably a performance hit */
     out = xrealloc(out, strlen(out)+1);  /* dealloc what we don't need */
+#endif
     return (out);
 }
 
@@ -237,25 +239,14 @@ static char *make_safe_uri(char *uri)
     assert(pos <= urilen);
     out[pos] = '\0';
 
+#if 0
+    /* don't really need to do this and it's probably a performance hit: */
     /* shorten buffer if necessary */
     if (pos != urilen) out = xrealloc(out, strlen(out)+1);
-
+#endif
     return (out);
 }
 
-
-
-/* ---------------------------------------------------------------------------
- * Make the specified socket non-blocking.
- */
-static void nonblock_socket(const int sock)
-{
-    if (fcntl(sock, F_SETFL, O_NONBLOCK) == -1)
-        err(1, "fcntl() to set O_NONBLOCK");
-}
-
-
-
 /* ---------------------------------------------------------------------------
  * Allocate and initialize an empty connection.
  */
@@ -318,7 +309,7 @@ static void accept_connection(void)
         /* else */ err(1, "accept()");
     }
 
-    nonblock_socket(sock);
+    fd_set_nonblock(sock);
 
     /* allocate and initialise struct connection */
     conn = new_connection();
@@ -840,7 +831,6 @@ void http_init(const in_addr_t bindaddr, const unsigned short bindport,
 
 /* ---------------------------------------------------------------------------
  * Set recv/send fd_sets and calculate timeout length.
- * Returns 1 if you need to use the timeout, 0 otherwise.
  */
 void
 http_fd_set(fd_set *recv_set, fd_set *send_set, int *max_fd,
@@ -848,10 +838,7 @@ http_fd_set(fd_set *recv_set, fd_set *send_set, int *max_fd,
 {
     struct connection *conn, *next;
     time_t now = time(NULL);
-
-    *need_timeout = 0;
-    timeout->tv_sec = idletime;
-    timeout->tv_usec = 0;
+    int minidle = idletime + 1;
 
     #define MAX_FD_SET(sock, fdset) do { \
         FD_SET(sock, fdset); *max_fd = max(*max_fd, sock); } while(0)
@@ -869,12 +856,9 @@ http_fd_set(fd_set *recv_set, fd_set *send_set, int *max_fd,
             conn->state = DONE;
         }
 
-        /* Set timeout. */
+        /* Connections that need a timeout. */
         if (conn->state != DONE)
-        {
-            *need_timeout = 1;
-            timeout->tv_sec = min(timeout->tv_sec, (idletime - idlefor));
-        }
+            minidle = min(minidle, (idletime - idlefor));
 
         switch (conn->state)
         {
@@ -898,6 +882,13 @@ http_fd_set(fd_set *recv_set, fd_set *send_set, int *max_fd,
         }
     }
     #undef MAX_FD_SET
+
+    /* Only set timeout if cap hasn't already. */
+    if ((*need_timeout == 0) && (minidle <= idletime)) {
+        *need_timeout = 1;
+        timeout->tv_sec = minidle;
+        timeout->tv_usec = 0;
+    }
 }
 
 
