@@ -89,6 +89,9 @@ function poll() {
     g[0].getElementsByTagName("e"));
   }
   document.getElementById("graph_reload").innerHTML = "reload graphs";
+  head = xh.responseXML.childNodes[0];
+  document.getElementById("tb").innerHTML = head.getAttribute("tb");
+  document.getElementById("tp").innerHTML = head.getAttribute("tp");
  }
 }
 
